/// <reference types="node" />
import { Merge } from 'type-fest';
import * as errors from './errors';
import { Options, Defaults, Response, CancelableRequest, URLOrOptions, HandlerFunction, ExtendedOptions } from './utils/types';
import { ProxyStream } from './as-stream';
import { Hooks } from './known-hook-events';
export declare type HTTPAlias = 'get' | 'post' | 'put' | 'patch' | 'head' | 'delete';
export declare type ReturnStream = (url: string | Options & {
    isStream: true;
}, options?: Options & {
    isStream: true;
}) => ProxyStream;
export declare type GotReturn = ProxyStream | CancelableRequest<Response>;
declare type OptionsOfDefaultResponseBody = Options & {
    isStream?: false;
    resolveBodyOnly?: false;
    responseType?: 'default';
};
declare type OptionsOfTextResponseBody = Options & {
    isStream?: false;
    resolveBodyOnly?: false;
    responseType: 'text';
};
declare type OptionsOfJSONResponseBody = Options & {
    isStream?: false;
    resolveBodyOnly?: false;
    responseType: 'json';
};
declare type OptionsOfBufferResponseBody = Options & {
    isStream?: false;
    resolveBodyOnly?: false;
    responseType: 'buffer';
};
declare type ResponseBodyOnly = {
    resolveBodyOnly: true;
};
interface GotFunctions {
    (url: string | OptionsOfDefaultResponseBody, options?: OptionsOfDefaultResponseBody): CancelableRequest<Response>;
    (url: string | OptionsOfTextResponseBody, options?: OptionsOfTextResponseBody): CancelableRequest<Response<string>>;
    (url: string | OptionsOfJSONResponseBody, options?: OptionsOfJSONResponseBody): CancelableRequest<Response<object>>;
    (url: string | OptionsOfBufferResponseBody, options?: OptionsOfBufferResponseBody): CancelableRequest<Response<Buffer>>;
    (url: string | OptionsOfDefaultResponseBody & ResponseBodyOnly, options?: OptionsOfDefaultResponseBody & ResponseBodyOnly): CancelableRequest<any>;
    (url: string | OptionsOfTextResponseBody & ResponseBodyOnly, options?: OptionsOfTextResponseBody & ResponseBodyOnly): CancelableRequest<string>;
    (url: string | OptionsOfJSONResponseBody & ResponseBodyOnly, options?: OptionsOfJSONResponseBody & ResponseBodyOnly): CancelableRequest<object>;
    (url: string | OptionsOfBufferResponseBody & ResponseBodyOnly, options?: OptionsOfBufferResponseBody & ResponseBodyOnly): CancelableRequest<Buffer>;
    (url: string | Options & {
        isStream: true;
    }, options?: Options & {
        isStream: true;
    }): ProxyStream;
}
export interface Got extends Merge<Record<HTTPAlias, GotFunctions>, GotFunctions> {
    stream: GotStream;
    defaults: Defaults | Readonly<Defaults>;
    GotError: typeof errors.GotError;
    CacheError: typeof errors.CacheError;
    RequestError: typeof errors.RequestError;
    ReadError: typeof errors.ReadError;
    ParseError: typeof errors.ParseError;
    HTTPError: typeof errors.HTTPError;
    MaxRedirectsError: typeof errors.MaxRedirectsError;
    UnsupportedProtocolError: typeof errors.UnsupportedProtocolError;
    TimeoutError: typeof errors.TimeoutError;
    CancelError: typeof errors.CancelError;
    extend(...instancesOrOptions: Array<Got | ExtendedOptions>): Got;
    mergeInstances(parent: Got, ...instances: Got[]): Got;
    mergeOptions<T extends Options>(...sources: T[]): T & {
        hooks: Partial<Hooks>;
    };
}
export interface GotStream extends Record<HTTPAlias, ReturnStream> {
    (url: URLOrOptions, options?: Options): ProxyStream;
}
export declare const defaultHandler: HandlerFunction;
declare const create: (defaults: Defaults) => Got;
export default create;
